@@ -32,7 +32,9 @@ For the name keyword a '*' is allowed on both sides
 For content '*' can occur on both sides, but does not match whitespace
 '''
 
+# TODO keyword for recent changes "changed>=date" - see KQl for inspiration
 # TODO keyword for deadlinks, keyword for pages with no content
+#     "no:links" "no:content" ?
 
 # Queries are parsed into trees of groups of search terms
 # Terms have a keyword and a string to look for
@@ -46,7 +48,7 @@ For content '*' can occur on both sides, but does not match whitespace
 import re
 import logging
 
-from zim.parsing import split_quoted_strings, unescape_quoted_string, Re
+from zim.parsing import Re, unescape_string
 from zim.notebook import Path, \
 	PageNotFoundError, IndexNotFoundError, \
 	LINK_DIR_BACKWARD, LINK_DIR_FORWARD
@@ -115,6 +117,48 @@ class QueryGroup(list):
 			self[:] = terms
 
 
+_word_re = re.compile(r'''
+	(	'(\\'|[^'])*' |  # single quoted word
+		"(\\"|[^"])*" |  # double quoted word
+		[^\s'"]+         # word without spaces
+	)''', re.X)
+
+
+def split_quoted_strings(string):
+	'''Split a word list respecting quotes, does not remove the quotes
+
+	Allow both double and single quotes
+
+	This function always expect full words to be quoted, even if quotes
+	appear in the middle of a word, they are considered word
+	boundries.
+	'''
+	string = string.strip()
+	words = []
+	m = _word_re.match(string)
+	while m:
+		words.append(m.group(0))
+		i = m.end()
+		string = string[i:].lstrip()
+		m = _word_re.match(string)
+
+	if string:
+		words += string.split() # unmatched quote ?
+
+	return [w for w in words if w]
+
+
+def unescape_quoted_string(string):
+	'''Removes quotes from a string and unescapes embedded quotes
+	@returns: string
+	'''
+	if not string:
+		return string
+	elif string[0] in ('"', "'") and string[-1] == string[0]:
+		string = string[1:-1]
+	return unescape_string(string)
+
+
 class Query(object):
 	'''This class wraps a query as typed by the user. It parses the
 	query into a tree of QueryGroup and QueryTerm objects. The 'root'
@@ -125,10 +169,11 @@ class Query(object):
 	def __init__(self, string):
 		self.string = string
 		self.root = self._parse_query(string)
+		self.find_input = self._generate_find_input()
 
 	def _parse_query(self, string):
 		# First do a raw tokenizer
-		words = split_quoted_strings(string, unescape=False, strict=False)
+		words = split_quoted_strings(string)
 		tokens = []
 		while words:
 			if operators_re.match(words[0]):
@@ -198,19 +243,31 @@ class Query(object):
 		#~ print root
 		return root
 
-	@property
-	def simple_match(self):
-		'''Used to determine a simple matching string to be used
-		in the find method in the pageview. Used by L{SearchDialog}
-		to set the L{PageView} find string to highligh matches in the page.
-		'''
-		# TODO make this return a list with positive terms for content
-		# if find supports an OR operator, highlight them all
-		if len(self.root) == 1 and isinstance(self.root[0], QueryTerm) \
-		and self.root[0].keyword in ('content', 'contentorname'):
-			return self.root[0].string
+	def _generate_find_input(self):
+		# parse query and format as a string or regex for the pageview "find"
+		# function - used to highlight matches in the pageview
+		strings = list(self._walk_text_content(self.root))
+		if not strings:
+			return None, None
+		elif len(strings) == 1:
+			return strings[0], False
 		else:
-			return None
+			return '|'.join(re.escape(s) for s in strings if s), True
+
+	def _walk_text_content(self, group):
+		for member in group:
+			if isinstance(member, QueryGroup):
+				for s in self._walk_text_content(member): # recurs
+					yield s
+			else: # QueryTerm
+				if member.inverse: # OPERATOR_NOT
+					pass
+				elif member.keyword in ('content', 'contentorname'):
+					yield member.string.strip('*') # strip "*" for partial matches
+				elif member.keyword == 'tag':
+					yield '@' + member.string.lstrip('@').strip('*')
+				else:
+					pass # other terms select pages, but no (easy) match in the page
 
 
 class PageSelection(set):
@@ -495,16 +552,18 @@ class SearchSelection(PageSelection):
 			term.content_regex = self._content_regex(term.string)
 			# term.name_regex already defined in _process_from_index
 
+		def page_generator(paths):
+			for path in paths:
+				try:
+					yield self.notebook.get_page(path)
+				except:
+					logger.exception('Exception opening: %s', path)
+					continue
+
 		if scope:
-			def page_generator():
-				for path in scope:
-					try:
-						yield self.notebook.get_page(path)
-					except PageNotFoundError:
-						pass
-			generator = page_generator()
+			generator = page_generator(scope)
 		else:
-			generator = list(map(self.notebook.get_page, self.notebook.pages.walk()))
+			generator = page_generator(self.notebook.pages.walk())
 
 		if results is None:
 			results = SearchSelection(None)
@@ -514,7 +573,7 @@ class SearchSelection(PageSelection):
 			try:
 				tree = page.get_parsetree()
 			except:
-				logger.exception('Exception while reading: %s', page)
+				logger.exception('Exception reading: %s', page)
 				continue
 
 			if tree is None:
